package test;


import java.awt.color.ColorSpace;
import java.awt.image.BufferedImage;
import java.awt.image.DataBuffer;
import java.awt.image.DirectColorModel;
import java.io.File;
import java.io.IOException;
import java.nio.ByteBuffer;

import javax.imageio.ImageIO;

import org.eclipse.jetty.websocket.api.Session;
import org.eclipse.jetty.websocket.api.annotations.OnWebSocketMessage;
import org.eclipse.jetty.websocket.api.annotations.WebSocket;

import com.goldencode.p2j.ui.client.gui.driver.ImageWrapper;
import com.goldencode.p2j.ui.client.gui.driver.PaintPrimitives;
import com.goldencode.p2j.ui.client.gui.driver.PaintStructure;


@WebSocket(maxIdleTime = 900000, maxTextMessageSize = 4096, maxBinaryMessageSize = 32768)
public class ImageTransferProtocol
{
   private final VirtualScreen virtualScreen;
   private final DrawImageOp drawImageOp;

   public ImageTransferProtocol(DirectColorModel colorModel)
   {
      super();
      virtualScreen = new VirtualScreenImpl(colorModel, 800, 600);
      drawImageOp = new DrawImageOp(virtualScreen);
   }
   public ImageTransferProtocol()
   {
      this(new DirectColorModel(ColorSpace.getInstance(ColorSpace.CS_sRGB),
               32, 0xFF, 0xFF00, 0xFF0000, 0, true, DataBuffer.TYPE_INT));
   }

   private byte[] loadBinariesImage(String relativePath)
   {
      try
      {
         BufferedImage image = ImageIO.read(new File(relativePath));
         int width = image.getWidth();
         int height = image.getHeight();
         virtualScreen.drawImage(image, 0, 0);
         byte[] bytes = new RawEncoder().packToBinaries(virtualScreen, 0, 0, width, height);
         
         return bytes;
      }
      catch (IOException e)
      {
         e.printStackTrace();
      }
      return null;
   }
   
   @OnWebSocketMessage
   public void onMessage(Session session, byte[] request, int offset, int length)
   {
      String imageRelativePath = "";
      try
      {
         imageRelativePath = new String(request,"ISO-8859-1");
         BufferedImage image = ImageIO.read(new File(imageRelativePath));
         PaintStructure ps = new PaintStructure(PaintPrimitives.DRAW_IMAGE);
         int w = image.getWidth();
         int h = image.getHeight();
         
         ps.xOffset = (virtualScreen.getWidth()  - w) >> 1;
         ps.yOffset = (virtualScreen.getHeight() - h) >> 1;
         ps.width = w;
         ps.height = h;
         ps.img = new ImageWrapper(image, ps.width, ps.height);
         byte[] message = drawImageOp.execute(ps);
         if (session.isOpen()) {
            session.getRemote().sendBytes(ByteBuffer.wrap(message));
         }
      }
      catch (IOException e)
      {
         e.printStackTrace();
      }
      
      System.out.println("on request [" + imageRelativePath + "]");
         
   }

}
