package test;

import java.io.Externalizable;
import java.io.IOException;
import java.io.ObjectInput;
import java.io.ObjectOutput;

/**
 * 
 * The following descriptions of protocol messages use the basic types U8, U16, U32,
 * S8, S16, S32. These represent respectively 8, 16 and 32-bit unsigned integers and
 * 8, 16 and 32-bit signed integers. All multiple byte integers (other than pixel values
 * themselves) are in big endian order (most significant byte first).
 * 
 */
public final class PixelFormat implements Externalizable
{
   
   private int bitsPerPixel;//U8
   private int depth;//U8
   private boolean bigEndianFlag;//U8
   private boolean trueColourFlag;//U8
   private int redMax;//U16
   private int greenMax;//U16
   private int blueMax;//U16
   private int redShift;//U8
   private int greenShift;//U8
   private int blueShift;//U8
   
   /**
    * @param bitsPerPixel
    * @param depth
    * @param bigEndianFlag
    * @param trueColourFlag
    * @param redMax
    * @param greenMax
    * @param blueMax
    * @param redShift
    * @param greenShift
    * @param blueShift
    */
   public PixelFormat(int bitsPerPixel, int depth, boolean bigEndianFlag, boolean trueColourFlag,
            int redMax, int greenMax, int blueMax, int redShift, int greenShift, int blueShift)
   {
      this.bitsPerPixel = bitsPerPixel;
      this.depth = depth;
      this.bigEndianFlag = bigEndianFlag;
      this.trueColourFlag = trueColourFlag;
      this.redMax = redMax;
      this.greenMax = greenMax;
      this.blueMax = blueMax;
      this.redShift = redShift;
      this.greenShift = greenShift;
      this.blueShift = blueShift;
   }

   /**
    * Filled with default values
    */
   public PixelFormat()
   {
      this(32, 24, false, true, 255, 255, 255, 16, 8, 0);
   }

   @Override
   public void writeExternal(ObjectOutput out) throws IOException
   {
      out.writeByte(bitsPerPixel);//U8
      out.writeByte(depth);//U8
      out.writeBoolean(bigEndianFlag);//U8
      out.writeBoolean(trueColourFlag);//U8
      out.writeShort(redMax);//U16
      out.writeShort(greenMax);//U16
      out.writeShort(blueMax);//U16
      out.writeByte(redShift);//U8
      out.writeByte(greenShift);//U8
      out.writeByte(blueShift);//U8
      out.writeByte(0);//padding U8
      out.writeByte(0);//U8
      out.writeByte(0);//U8
   }
   
   @Override
   public void readExternal(ObjectInput in) throws IOException, ClassNotFoundException
   {
      bitsPerPixel = in.readByte();
      depth = in.readByte();
      bigEndianFlag = in.readBoolean();
      trueColourFlag = in.readBoolean();
      redMax = in.readUnsignedShort();
      greenMax = in.readUnsignedShort();
      blueMax = in.readUnsignedShort();
      redShift = in.readUnsignedByte();
      greenShift = in.readUnsignedByte();
      blueShift = in.readUnsignedByte();
      in.readByte();
      in.readByte();
      in.readByte();
   }

   public int getBitsPerPixel()
   {
      return bitsPerPixel;
   }

   public void setBitsPerPixel(int bitsPerPixel)
   {
      this.bitsPerPixel = bitsPerPixel;
   }

   public int getDepth()
   {
      return depth;
   }

   public void setDepth(int depth)
   {
      this.depth = depth;
   }

   public boolean isBigEndianFlag()
   {
      return bigEndianFlag;
   }

   public void setBigEndianFlag(boolean bigEndianFlag)
   {
      this.bigEndianFlag = bigEndianFlag;
   }

   public boolean isTrueColourFlag()
   {
      return trueColourFlag;
   }

   public void setTrueColourFlag(boolean trueColourFlag)
   {
      this.trueColourFlag = trueColourFlag;
   }

   public int getRedMax()
   {
      return redMax;
   }

   public void setRedMax(int redMax)
   {
      this.redMax = redMax;
   }

   public int getGreenMax()
   {
      return greenMax;
   }

   public void setGreenMax(int greenMax)
   {
      this.greenMax = greenMax;
   }

   public int getBlueMax()
   {
      return blueMax;
   }

   public void setBlueMax(int blueMax)
   {
      this.blueMax = blueMax;
   }

   public int getRedShift()
   {
      return redShift;
   }

   public void setRedShift(int redShift)
   {
      this.redShift = redShift;
   }

   public int getGreenShift()
   {
      return greenShift;
   }

   public void setGreenShift(int greenShift)
   {
      this.greenShift = greenShift;
   }

   public int getBlueShift()
   {
      return blueShift;
   }

   public void setBlueShift(int blueShift)
   {
      this.blueShift = blueShift;
   }

}
