package test;

import java.awt.image.DirectColorModel;

public class RawEncoder implements PixelsEncoder
{

   private static int getShiftFromMask(int mask )
   {
      int shift = 0;
      while( ( mask & 1 ) == 0 )
      {
         shift++;
         mask >>= 1;
      }
      
      return shift;
   }
   
   @Override
   public byte[] packToBinaries(VirtualScreen screen, int x, int y,
            int w, int h)
   {
      byte[] bytes = null;
      // Encode as bytes
      int b = 0;
      int i = 0;
      int s = 0;
      int pixel;
      int size = w * h;
      int scanline = screen.getWidth();
      int jump = scanline - w;
      int p = y * scanline + x;
      int[] pixels = screen.getPixels();
      DirectColorModel colorModel = screen.getColorModel();
      int redShift = getShiftFromMask(colorModel.getRedMask());
      int greenShift = getShiftFromMask(colorModel.getGreenMask());
      int blueShift = getShiftFromMask(colorModel.getBlueMask());
      int alphaShift = 24;//getShiftFromMask(colorModel.getAlphaMask());
      switch( colorModel.getPixelSize() )
      {
      case 32:
         bytes = new byte[ size << 2 ];
         for( ; i < size; i++, s++, p++ )
         {
            if( s == w )
            {
               s = 0;
               p += jump;
            }
            pixel = pixels[p];
            bytes[b++] = (byte)( (pixel >> redShift) & 0xFF );
            bytes[b++] = (byte)( ( pixel >> greenShift ) & 0xFF );
            bytes[b++] = (byte)( ( pixel >> blueShift ) & 0xFF );
            bytes[b++] = (byte)( ( pixel >> alphaShift ) & 0xFF );
         }
         break;
      case 16:
         bytes = new byte[ size << 1 ];
         for( ; i < size; i++, s++, p++ )
         {
            if( s == w )
            {
               s = 0;
               p += jump;
            }
            pixel = pixels[p];
            bytes[b++] = (byte)( pixel & 0xFF );
            bytes[b++] = (byte)( ( pixel >> 8 ) & 0xFF );
         }
         break;
      case 8:
         bytes = new byte[ size ];
         for( ; i < size; i++, s++, p++ )
         {
            if( s == w )
            {
               s = 0;
               p += jump;
            }
            bytes[i] = (byte) ( pixels[p] );
         }
         break;
      }
      
      return bytes;
   }

}
