package test;

import java.awt.Color;
import java.awt.Graphics2D;
import java.awt.Image;
import java.awt.image.BufferedImage;
import java.awt.image.DataBuffer;
import java.awt.image.DataBufferInt;
import java.awt.image.DirectColorModel;
import java.awt.image.Raster;
import java.awt.image.SampleModel;
import java.awt.image.SinglePixelPackedSampleModel;
import java.awt.image.WritableRaster;
import java.nio.ByteBuffer;
import java.nio.IntBuffer;
import java.security.MessageDigest;
import java.security.NoSuchAlgorithmException;


class VirtualScreenImpl implements VirtualScreen
{

   private final DirectColorModel colorModel;
   private final int width;
   private final int height;
   private final int[] pixels;
   private final BufferedImage screen;

   /**
    * @param colorModel
    * @param width
    * @param height
    */
   public VirtualScreenImpl(DirectColorModel colorModel, int width, int height)
   {
      this.colorModel = colorModel;
      this.width = width;
      this.height = height;
      this.pixels = new int[width*height];
      final DataBuffer data = new DataBufferInt(pixels, pixels.length);
      final SampleModel sampleModel = new SinglePixelPackedSampleModel(
               DataBuffer.TYPE_INT, width,  height, colorModel.getMasks());
      final WritableRaster raster = Raster.createWritableRaster(sampleModel, data, null);
      this.screen = new BufferedImage(colorModel, raster, true, null);
   }

   @Override
   public int[] getPixels()
   {
      return pixels;
   }

   @Override
   public int getWidth()
   {
      return width;
   }

   @Override
   public int getHeight()
   {
      return height;
   }

   @Override
   public DirectColorModel getColorModel()
   {
      return colorModel;
   }

   private byte[] digest(Image image)
   {
      byte[] md5ImageHash = null;
      try
      {
         MessageDigest md = MessageDigest.getInstance("MD5");
         int imageLength = image.getWidth(null) * image.getHeight(null);
         ByteBuffer byteBuffer = ByteBuffer.allocate(imageLength * 4);
         IntBuffer intBuffer = byteBuffer.asIntBuffer();
         intBuffer.put(pixels, 0, imageLength);
         md.update(byteBuffer);
         md5ImageHash = md.digest();
      }
      catch (NoSuchAlgorithmException e)
      {
         e.printStackTrace();
      }
      return md5ImageHash;
   }

   @Override
   public byte[] drawImage(Image image, int x, int y)
   {
      Graphics2D g2 = screen.createGraphics();
      g2.setColor(new Color(0,0,0));
      g2.fillRect(0, 0, width, height);
      g2.drawImage(image, x, y, null);
      g2.dispose();
      return digest(image);
   }

}
